import deepEqual from "deep-equal";
import { ATTRIBUTE_NAMES, TAG_NAMES } from "./HelmetConstants.js";
import { generateTitleAsReactComponent, convertElementAttributestoReactProps, generateTagsAsReactComponent, handleClientStateChange, reducePropsToState } from "./HelmetUtils";

var getComponentForTag = function getComponentForTag(type, tags, encode, typeComponents) {
    switch (type) {
        case TAG_NAMES.TITLE:
            return generateTitleAsReactComponent(type, tags.title, tags.titleAttributes, encode);
        case ATTRIBUTE_NAMES.BODY:
        case ATTRIBUTE_NAMES.HTML:
            return convertElementAttributestoReactProps(tags);
        default:
            return generateTagsAsReactComponent(type, tags, typeComponents);
    }
};

var mapStateToComponents = function mapStateToComponents(_ref) {
    var baseTag = _ref.baseTag,
        bodyAttributes = _ref.bodyAttributes,
        encode = _ref.encode,
        htmlAttributes = _ref.htmlAttributes,
        linkTags = _ref.linkTags,
        metaTags = _ref.metaTags,
        noscriptTags = _ref.noscriptTags,
        scriptTags = _ref.scriptTags,
        styleTags = _ref.styleTags,
        _ref$title = _ref.title,
        title = _ref$title === undefined ? "" : _ref$title,
        titleAttributes = _ref.titleAttributes,
        typeComponents = _ref.typeComponents;
    return {
        base: getComponentForTag(TAG_NAMES.BASE, baseTag, encode, typeComponents),
        bodyAttributes: getComponentForTag(ATTRIBUTE_NAMES.BODY, bodyAttributes, encode),
        htmlAttributes: getComponentForTag(ATTRIBUTE_NAMES.HTML, htmlAttributes, encode),
        link: getComponentForTag(TAG_NAMES.LINK, linkTags, encode, typeComponents),
        meta: getComponentForTag(TAG_NAMES.META, metaTags, encode, typeComponents),
        noscript: getComponentForTag(TAG_NAMES.NOSCRIPT, noscriptTags, encode, typeComponents),
        script: getComponentForTag(TAG_NAMES.SCRIPT, scriptTags, encode, typeComponents),
        style: getComponentForTag(TAG_NAMES.STYLE, styleTags, encode, typeComponents),
        title: getComponentForTag(TAG_NAMES.TITLE, { title: title, titleAttributes: titleAttributes }, encode)
    };
};

export { reducePropsToState, handleClientStateChange, mapStateToComponents, deepEqual };